// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#if 0 // The public interface of TCPClientSocket

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class TCPConnector;

//------------------------------------------------------------------------------

/**
 * Client socket that connects to a server socket over TCP.
 *
 * The server socket's IP address and port should be given.
 */
class TCPClientSocket : public BufferedStream
{
public:
    /**
     * Construct the client socket with the given address and port number.
     *
     * @param addr the address of the server socket
     * @param port the address of the server socket
     * @param waiter the waiter to use for waiting a possible event
     * occuring on the socket. If given, the socket will be
     * non-blocking. Otherwise, it will be blocking, and the connect()
     * function of the Connector might block as well as the reading
     * and writing buffers.
     * @param readingCapacity the capacity of the reading buffer
     * @param writingCapacity the capacity of the writing buffer
     */
    TCPClientSocket(struct in_addr address, unsigned short port,
                    Waiter* waiter = 0,
                    size_t readingCapacity = DEFAULT_CAPACITY,
                    size_t writingCapacity = DEFAULT_CAPACITY);

    /**
     * Construct the client socket with the given address and port number.
     *
     * @param addr the address of the server socket expressed as a string
     * @param port the address of the server socket
     * @param waiter the waiter to use for waiting a possible event
     * occuring on the socket. If given, the socket will be
     * non-blocking. Otherwise, it will be blocking, and the connect()
     * function of the Connector might block as well as the reading
     * and writing buffers.
     * @param readingCapacity the capacity of the reading buffer
     * @param writingCapacity the capacity of the writing buffer
     */
    TCPClientSocket(const char* address, unsigned short port,
                    Waiter* waiter = 0,
                    size_t readingCapacity = DEFAULT_CAPACITY,
                    size_t writingCapacity = DEFAULT_CAPACITY);

    /**
     * Get the connector. Use this object to connect to another socket.
     */
    TCPConnector& getConnector();
};

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

#endif // 0, The public interface of LocalClientSocket

//------------------------------------------------------------------------------

#include "hu/varadiistvan/scpl/config.h"

//------------------------------------------------------------------------------

#if TARGET_API_POSIX

#include "_posix/TCPClientSocket.h"

//------------------------------------------------------------------------------

#elif TARGET_API_WIN32 || TARGET_API_WIN64

#include "_win32/TCPClientSocket.h"

//------------------------------------------------------------------------------

#else

#error "Not implemented for this target!"

#endif

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
